from ..compatibility import MagicMock

from django.template.loader import render_to_string
from django.test import TestCase

from vcs_info_panel.clients.git import GitClient
from vcs_info_panel.panels import GitInfoPanel


class GitInfoPanelTestCase(TestCase):
    def setUp(self):
        self.toolbar = MagicMock()
        self.panel = GitInfoPanel(self.toolbar)

    def test_client_class(self):
        self.assertEqual(self.panel.client_class, GitClient)

    def test_content(self):
        rendered = render_to_string('git.html')
        self.assertEqual(self.panel.client.base_command, 'git')
        self.assertEqual(self.panel.content, rendered)



from dateutil.parser import parse
from .base import BaseVCSClient, returns_on_fail


class GitClient(BaseVCSClient):
    base_command = 'git'

    @returns_on_fail(False)
    def is_repository(self):
        is_work_dir = self._execute_vcs('rev-parse', '--is-inside-work-tree')
        return is_work_dir.startswith('true')

    @returns_on_fail(None)
    def get_hash(self):
        return self._execute_vcs('rev-parse', 'HEAD')

    @returns_on_fail(None)
    def get_short_hash(self):
        return self._execute_vcs('rev-parse', '--short', 'HEAD')

    @returns_on_fail(None)
    def get_current_branch_name(self):
        return self._execute_vcs('rev-parse', '--abbrev-ref', 'HEAD')

    @returns_on_fail(None)
    def get_author_name(self):
        return self._execute_vcs_show('%an')

    @returns_on_fail(None)
    def get_author_email(self):
        return self._execute_vcs_show('%ae')

    @returns_on_fail(None)
    def get_author_info(self):
        return self._execute_vcs_show('%an<%ae>')

    @returns_on_fail(None)
    def get_committer_name(self):
        return self._execute_vcs_show('%cn')

    @returns_on_fail(None)
    def get_committer_email(self):
        return self._execute_vcs_show('%ce')

    @returns_on_fail(None)
    def get_committer_info(self):
        return self._execute_vcs_show('%cn<%ce>')

    @returns_on_fail(None)
    def get_date(self):
        iso_format = self._execute_vcs_show('%ci')
        try:
            return parse(iso_format)
        except:
            return None

    @returns_on_fail(None)
    def get_subject(self):
        return self._execute_vcs_show('%s')

    @returns_on_fail(None)
    def get_body(self):
        return self._execute_vcs_show('%b')

    @returns_on_fail(None)
    def get_message(self):
        subject = self.get_subject()
        body = self.get_body()
        if not body:
            return subject
        return '\n'.join([subject, body])

    def _execute_vcs_show(self, format_text, hash='HEAD'):
        return self._execute_vcs('show', '--quiet', '--format={}'.format(format_text), hash).strip()



import subprocess
import pytz
import datetime
from ..compatibility import patch
from django.test import TestCase
from vcs_info_panel.clients.git import GitClient


def without_git_repository(*commands):
    def decorator(func):
        def inner(*args, **kwargs):
            with patch('subprocess.check_output') as _check_output:
                _check_output.side_effect = subprocess.CalledProcessError(128,
                                                                          commands,
                                                                          'fatal: Not a git repository (or any of the parent directories): .git')
                return func(*args, **kwargs)
        return inner
    return decorator


class GitClientTestCase(TestCase):
    def setUp(self):
        self.client = GitClient()

    def test_base_command(self):
        self.assertEqual(self.client.base_command, 'git')

    def test_is_repository_with_repository(self):
        with patch('subprocess.check_output') as _check_output:
            _check_output.return_value = b'true'
            self.assertEqual(self.client.is_repository(), True)
            _check_output.assert_called_once_with(['git', 'rev-parse', '--is-inside-work-tree'])

    @without_git_repository('git', 'rev-parse', '--is-inside-work-tree')
    def test_is_repository_without_repository(self):
        self.assertEqual(self.client.is_repository(), False)

    def test_get_short_hash(self):
        with patch('subprocess.check_output') as _check_output:
            _check_output.return_value = b'3218766'
            self.assertEqual(self.client.get_short_hash(), '3218766')
            _check_output.assert_called_once_with(['git', 'rev-parse', '--short', 'HEAD'])

    @without_git_repository('git', 'rev-parse', 'HEAD')
    def test_get_short_hash_without_repository(self):
        self.assertEqual(self.client.get_short_hash(), None)

    def test_get_hash(self):
        with patch('subprocess.check_output') as _check_output:
            _check_output.return_value = b'32187666c7b6b7af6165f38aa8dbde4922d8423a'
            self.assertEqual(self.client.get_hash(), '32187666c7b6b7af6165f38aa8dbde4922d8423a')
            _check_output.assert_called_once_with(['git', 'rev-parse', 'HEAD'])

    @without_git_repository('git', 'rev-parse', 'HEAD')
    def test_get_hash_without_repository(self):
        self.assertEqual(self.client.get_hash(), None)

    def test_get_current_branch_name(self):
        with patch('subprocess.check_output') as _check_output:
            _check_output.return_value = b'master'
            self.assertEqual(self.client.get_current_branch_name(), 'master')
            _check_output.assert_called_once_with(['git', 'rev-parse', '--abbrev-ref', 'HEAD'])

    @without_git_repository('git', 'rev-parse', '--abbrev-ref', 'HEAD')
    def test_get_current_branch_name_without_repository(self):
        self.assertEqual(self.client.get_current_branch_name(), None)

    def test_get_author_name(self):
        with patch('subprocess.check_output') as _check_output:
            _check_output.return_value = b'giginet'
            self.assertEqual(self.client.get_author_name(), 'giginet')
            _check_output.assert_called_once_with(['git', 'show', '--quiet', '--format=%an', 'HEAD'])

    @without_git_repository('git', 'show', '--quiet', '--format=%an', 'HEAD')
    def test_get_author_name_without_repository(self):
        self.assertEqual(self.client.get_author_email(), None)

    def test_get_author_email(self):
        with patch('subprocess.check_output') as _check_output:
            _check_output.return_value = b'giginet@kawaz.org'
            self.assertEqual(self.client.get_author_email(), 'giginet@kawaz.org')
            _check_output.assert_called_once_with(['git', 'show', '--quiet', '--format=%ae', 'HEAD'])

    @without_git_repository('git', 'show', '--quiet', '--format=%ae', 'HEAD')
    def test_get_author_email_without_repository(self):
        self.assertEqual(self.client.get_author_email(), None)

    def test_get_author_info(self):
        with patch('subprocess.check_output') as _check_output:
            _check_output.return_value = b'giginet<giginet@kawaz.org>'
            self.assertEqual(self.client.get_author_info(), 'giginet<giginet@kawaz.org>')
            _check_output.assert_called_once_with(['git', 'show', '--quiet', '--format=%an<%ae>', 'HEAD'])

    @without_git_repository('git', 'show', '--quiet', '--format=%an<%ae>', 'HEAD')
    def test_get_author_info_without_repository(self):
        self.assertEqual(self.client.get_author_info(), None)

    def test_get_committer_name(self):
        with patch('subprocess.check_output') as _check_output:
            _check_output.return_value = b'giginet'
            self.assertEqual(self.client.get_committer_name(), 'giginet')
            _check_output.assert_called_once_with(['git', 'show', '--quiet', '--format=%cn', 'HEAD'])

    @without_git_repository('git', 'show', '--quiet', '--format=%cn', 'HEAD')
    def test_get_committer_name_without_repository(self):
        self.assertEqual(self.client.get_committer_email(), None)

    def test_get_committer_email(self):
        with patch('subprocess.check_output') as _check_output:
            _check_output.return_value = b'giginet@kawaz.org'
            self.assertEqual(self.client.get_committer_email(), 'giginet@kawaz.org')
            _check_output.assert_called_once_with(['git', 'show', '--quiet', '--format=%ce', 'HEAD'])

    @without_git_repository('git', 'show', '--quiet', '--format=%ce', 'HEAD')
    def test_get_committer_email_without_repository(self):
        self.assertEqual(self.client.get_committer_email(), None)

    def test_get_committer_info(self):
        with patch('subprocess.check_output') as _check_output:
            _check_output.return_value = b'giginet<giginet@kawaz.org>'
            self.assertEqual(self.client.get_committer_info(), 'giginet<giginet@kawaz.org>')
            _check_output.assert_called_once_with(['git', 'show', '--quiet', '--format=%cn<%ce>', 'HEAD'])

    @without_git_repository('git', 'show', '--quiet', '--format=%cn<%ce>', 'HEAD')
    def test_get_committer_info_without_repository(self):
        self.assertEqual(self.client.get_committer_info(), None)

    def test_get_date(self):
        with patch('subprocess.check_output') as _check_output:
            _check_output.return_value = b'2015-12-04 20:29:10 +0900'
            jst = pytz.timezone('Asia/Tokyo')
            self.assertEqual(self.client.get_date(), datetime.datetime(2015, 12, 4, 20, 29, 10, tzinfo=jst))
            _check_output.assert_called_once_with(['git', 'show', '--quiet', '--format=%ci', 'HEAD'])

    def test_get_date_with_invalid_return(self):
        with patch('subprocess.check_output') as _check_output:
            _check_output.return_value = b'2015-12-99'
            self.assertEqual(self.client.get_date(), None)
            _check_output.assert_called_once_with(['git', 'show', '--quiet', '--format=%ci', 'HEAD'])

    @without_git_repository('git', 'show', '--quiet', '--format=%ci', 'HEAD')
    def test_get_date_without_repository(self):
        self.assertEqual(self.client.get_date(), None)

    def test_get_message(self):
        with patch('subprocess.check_output') as _check_output:
            def check_output(commands):
                format = commands[-2]
                if format.endswith('%s'):
                    return b'Fix the issue'
                elif format.endswith('%b'):
                    return b'This commit will fix #10'
            _check_output.side_effect = check_output
            self.assertEqual(self.client.get_message(), 'Fix the issue\nThis commit will fix #10')

    def test_get_message_with_invalid_body(self):
        with patch('subprocess.check_output') as _check_output:
            def check_output(commands):
                format = commands[-2]
                if format.endswith('%s'):
                    return b'Fix the issue'
                elif format.endswith('%b'):
                    raise subprocess.CalledProcessError(128, ['git', 'show'])
            _check_output.side_effect = check_output
            self.assertEqual(self.client.get_message(), 'Fix the issue')

    @without_git_repository('git', 'show', '--quiet', '--format=%b', 'HEAD')
    def test_get_message_without_repository(self):
        self.assertEqual(self.client.get_message(), None)



import os
import subprocess


def returns_on_fail(return_value):
    def decorator(func):
        def inner(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except subprocess.CalledProcessError:
                return return_value
        return inner
    return decorator


class BaseVCSClient(object):
    @property
    def base_command(self):
        raise NotImplementedError

    def is_repository(self):
        raise NotImplementedError

    def get_hash(self):
        raise NotImplementedError

    def get_short_hash(self):
        raise NotImplementedError

    def get_current_branch_name(self):
        raise NotImplementedError

    def get_author_name(self):
        raise NotImplementedError

    def get_author_email(self):
        raise NotImplementedError

    def get_author_info(self):
        raise NotImplementedError

    def get_committer_name(self):
        raise NotImplementedError

    def get_committer_email(self):
        raise NotImplementedError

    def get_committer_info(self):
        raise NotImplementedError

    def get_date(self):
        raise NotImplementedError

    def get_message(self):
        raise NotImplementedError

    def _get_cwd(self):
        return os.getcwd()

    def _execute_vcs(self, main, *options):
        commands = [self.base_command, main] + list(options)
        raw_data = subprocess.check_output(commands)
        return raw_data.decode()



import datetime
from ..compatibility import MagicMock, PropertyMock
from django.test import TestCase

from vcs_info_panel.panels import BaseVCSInfoPanel


class BaseVCSPanelTestCase(TestCase):
    def setUp(self):
        self.toolbar = MagicMock()
        self.panel = BaseVCSInfoPanel(self.toolbar)
        self.mock_client = MagicMock()
        self.original_client = type(self.panel).client
        type(self.panel).client = PropertyMock(return_value=self.mock_client)

    def tearDown(self):
        type(self.panel).client = self.original_client

    def test_template(self):
        self.mock_client.base_command = 'vcs_name'
        self.assertEqual(self.panel.template, 'vcs_name.html')

    def test_has_content_is_true(self):
        self.panel.client.is_repository = MagicMock(return_value=True)
        self.assertEqual(self.panel.has_content, True)

    def test_has_content_is_false(self):
        self.panel.client.is_repository = MagicMock(return_value=False)
        self.assertEqual(self.panel.has_content, False)

    def test_nav_subtitle(self):
        self.panel.client.get_short_hash = MagicMock(return_value='bba82a1')
        self.assertEqual(self.panel.nav_subtitle, 'bba82a1')

    def test_nav_subtitle_with_not_git_repository(self):
        self.panel.client.is_repository = MagicMock(return_value=False)
        self.assertEqual(self.panel.nav_subtitle, 'repository is not detected')

    def test_get_stats(self):
        self.toolbar.stats = {self.panel.panel_id: {'key': 'value'}}
        self.panel.client.is_repository = MagicMock(return_value=True)
        self.panel.client.get_short_hash = MagicMock(return_value='bba82a1')
        self.panel.client.get_hash = MagicMock(return_value='bba82a1877683b98bccdee4a22d5b6ff40aaa536')
        self.panel.client.get_author_name = MagicMock(return_value='giginet')
        self.panel.client.get_author_email = MagicMock(return_value='giginet@kawaz.org')
        self.panel.client.get_author_info = MagicMock(return_value='giginet<giginet@kawaz.org>')
        self.panel.client.get_committer_name = MagicMock(return_value='giginet2')
        self.panel.client.get_committer_email = MagicMock(return_value='giginet2@kawaz.org')
        self.panel.client.get_committer_info = MagicMock(return_value='giginet2<giginet2@kawaz.org>')
        self.panel.client.get_message = MagicMock(return_value='Fix issue')
        self.panel.client.get_date = MagicMock(return_value=datetime.datetime(2015, 1, 1))
        self.panel.client.get_current_branch_name = MagicMock(return_value='master')

        stats = self.panel.get_stats()
        self.assertEqual(stats['short_hash'], 'bba82a1')
        self.assertEqual(stats['hash'], 'bba82a1877683b98bccdee4a22d5b6ff40aaa536')
        self.assertEqual(stats['author'], 'giginet<giginet@kawaz.org>')
        self.assertEqual(stats['committer'], 'giginet2<giginet2@kawaz.org>')
        self.assertEqual(stats['message'], 'Fix issue')
        self.assertEqual(stats['updated_at'], datetime.datetime(2015, 1, 1))
        self.assertEqual(stats['branch_name'], 'master')
        self.assertEqual(stats['key'], 'value')

    def test_get_stats_without_repository(self):
        self.toolbar.stats = {self.panel.panel_id: {'key': 'value'}}
        self.panel.client.is_repository = MagicMock(return_value=False)
        self.assertEqual(self.panel.get_stats(), {'key': 'value'})



from debug_toolbar.panels import Panel
from django.utils.translation import ugettext_lazy as _


class BaseVCSInfoPanel(Panel):
    title = _('Revision')
    _cached_client = None

    @property
    def client_class(self):
        raise NotImplementedError

    @property
    def client(self):
        if not self._cached_client:
            self._cached_client = self.client_class()
        return self._cached_client

    @property
    def nav_subtitle(self):
        if self.client.is_repository():
            return self.client.get_short_hash()
        return 'repository is not detected'

    @property
    def has_content(self):
        return self.client.is_repository()

    @property
    def template(self):
        return '{}.html'.format(self.client.base_command)

    def get_stats(self):
        context = super(BaseVCSInfoPanel, self).get_stats()
        if self.client.is_repository():
            context.update({
                'short_hash': self.client.get_short_hash(),
                'hash': self.client.get_hash(),
                'author': self.client.get_author_info(),
                'committer': self.client.get_committer_info(),
                'message': self.client.get_message(),
                'updated_at': self.client.get_date(),
                'branch_name': self.client.get_current_branch_name()
            })
        return context



try:
    # Python 3 have mock in unittest
    from unittest.mock import MagicMock, PropertyMock, patch
except ImportError:
    from mock import MagicMock, PropertyMock, patch

from django.conf import settings
try:
    from django.test.runner import DiscoverRunner as TestRunnerBase
except ImportError:
    from django.test.simple import DjangoTestSuiteRunner as TestRunnerBase
settings.TESTING = False


class TestRunner(TestRunnerBase):
    def setup_test_environment(self, **kwargs):
        super(TestRunner, self).setup_test_environment(**kwargs)
        settings.TESTING = True

    def teardown_test_environment(self, **kwargs):
        super(TestRunner, self).teardown_test_environment(**kwargs)
        settings.TESTING = False



# coding=utf-8
import sys
from setuptools import setup, find_packages

NAME = 'django-debug-toolbar-vcs-info'
VERSION = '1.1.0'


def read(filename):
    import os
    BASE_DIR = os.path.dirname(__file__)
    filename = os.path.join(BASE_DIR, filename)
    with open(filename, 'r') as fi:
        return fi.read()


def readlist(filename):
    rows = read(filename).split("\n")
    rows = [x.strip() for x in rows if x.strip()]
    return list(rows)

# if we are running on python 3, enable 2to3 and
# let it use the custom fixers from the custom_fixers
# package.
extra = {}
if sys.version_info >= (3, 0):
    extra.update(
        use_2to3=True,
    )

setup(
    name=NAME,
    version=VERSION,
    description='A Django Debug Toolbar panel to show VCS info',
    long_description=read('README.rst'),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords='django django-debug-toolbar git vcs info revision hash panel',
    author='giginet',
    author_email='giginet.net@gmail.com',
    url='https://github.com/giginet/%s' % NAME,
    download_url='https://github.com/giginet/%s/tarball/master' % NAME,
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    package_data={
        '': ['README.rst',
             'requirements.txt',
             'requirements-test.txt'],
    },
    zip_safe=True,
    install_requires=readlist('requirements.txt'),
    test_suite='runtests.run_tests',
    tests_require=readlist('requirements-test.txt'),
    **extra
)
